from os import linesep

item_types = {
  'simple': '-',
  'bulleted': '*',
  'numbered': '#'
}

class ListItem:
  def __init__(self, text, item_type, depth=0):
    self.text = text
    self.item_type = item_type
    self.depth = depth
  
  def __str__(self):
    prefix = "".join([item_types[self.item_type] for _ in range(self.depth + 1)])
    return f'{prefix} {self.text}'

class List:
  def __init__(self, list_type='simple'):
    if list_type not in item_types:
      raise Exception(f'Invalid list type. The supported list types are [{",".join([list_type for list_type in item_types])}]')
  
    self.list_type = list_type
    self.items = []
  
  def append(self, item, item_type=None, depth=0):
    if depth < 0:
      raise Exception('List item depth cannot be negative')

    if not item_type:
      item_type = self.list_type
  
    self.items.append(ListItem(item, item_type, depth))
  
  def __str__(self):
    if not self.items:
      return ''
    
    return '\n'.join([str(item) for item in self.items])



from os import linesep

class Writer:
  def __init__(self, stdout):
    self.stdout = stdout
  
  def write(self, text=''):
    self.stdout.write(str(text))
  
  def write_line(self, text=''):
    self.stdout.write(f'{str(text)}{linesep}')
  
  def write_lines(self, lines):
    for line in lines:
      self.stdout.write(f'{str(line)}{linesep}')
  
  def write_heading(self, text, level=1):
    if level not in range(1, 7):
      raise Exception('Invalid heading level. Levels can only be in the inclusive range [1..6]')
    self.stdout.write(f'h{level}.{str(text)}{linesep}')
  
  def write_hrule(self):
    self.stdout.write('----')


def wrap(text, wrap_sequence):
  return f'{wrap_sequence}{text}{wrap_sequence}' if text else ''

def emphasis(text):
  return wrap(text, '_')

def strong(text):
  return wrap(text, '*')

def citation(text):
  return wrap(text, '??')

def strikethrough(text):
  return wrap(text, '-')

def underline(text):
  return wrap(text, '+')

def superscript(text):
  return wrap(text, '^')

def subscript(text):
  return wrap(text, '~')

def monospace(text):
  return '{{' + text + '}}' if text else ''

def block_quote(text):
  return f'bq. {text}' if text else ''

def colour(text, colour):
  return f'{{color:{colour}}}{text}{{color}}' if text and colour else ''


from .writer import Writer
from .list import List
from .table import Table
from .emphasis import (
  emphasis,
  strong,
  citation,
  strikethrough,
  underline,
  superscript,
  subscript,
  monospace,
  block_quote,
  colour
)


import setuptools

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
  name="confluence-wiki-generator",
  varsion="0.0.1",
  author="Cuan Clifford",
  author_email="cliffordcuan@gmail.com",
  description="A Confluence Wiki markup generator",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/cjclifford/confluence-wiki-generator",
  packages=setuptools.find_packages(),
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent"
  ],
  python_requires='>=3.8'
)


import itertools

class Table:
  def __init__(self, columns, vertical=False):
    self.vertical = vertical
    self.columns = columns
    self.rows = []

  def append(self, row):
    self.rows.append(row)

  def __str__(self):
    lines = []

    if self.vertical:
      rotated_table = itertools.zip_longest(*self.rows[::])
      for (column, vertical_row) in itertools.zip_longest(self.columns, rotated_table):
        lines.append(f'||{column}|{"|".join([item if item else " " for item in vertical_row])}|')
    else:
      lines.append(f'||{"||".join([column for column in self.columns])}||')
      for row in self.rows:
        lines.append(f'|{"|".join([item for item in row])}|')
    return '\n'.join(lines)
